var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { ConnectLogger, createLogMessage, } from "../logging";
import { SubscriptionManager, } from "../messaging/subscription";
import { createMetricMessage } from "../metric";
import { createRequestMessage, RequestManager, } from "../request";
import { ChannelManager, } from "./channel-manager";
import { ErrorService } from "./error";
import { HealthCheckManager, } from "./health-check";
import { ProxyConnectionStatusManager, } from "./proxy-connection";
export class Proxy {
    constructor(provider) {
        this.provider = provider;
        this.logger = new ConnectLogger({
            source: "core.proxy",
            provider,
            mixin: () => ({
                proxyType: this.proxyType,
                connectionId: this.connectionId,
            }),
        });
        this.requestManager = new RequestManager(provider);
        this.status = new ProxyConnectionStatusManager(provider);
        this.errorService = new ErrorService(provider);
        this.upstreamMessageQueue = [];
        this.connectionEstablished = false;
        this.isInitialized = false;
        this.subscriptions = new SubscriptionManager();
        this.connectionId = null;
        this.channelManager = new ChannelManager(provider, this.sendOrQueueMessageToSubject.bind(this));
        this.healthCheck = new HealthCheckManager({
            provider,
            sendHealthCheck: this.sendOrQueueMessageToSubject.bind(this),
            getUpstreamMessageOrigin: this.getUpstreamMessageOrigin.bind(this),
        });
    }
    init() {
        if (this.isInitialized)
            throw new Error("Proxy already initialized");
        this.isInitialized = true;
        this.initProxy();
    }
    request(namespace, command, data, origin) {
        const msg = createRequestMessage(namespace, command, data, origin !== null && origin !== void 0 ? origin : this.getUpstreamMessageOrigin());
        const resp = this.requestManager.processRequest(msg);
        this.sendOrQueueMessageToSubject(msg);
        return resp;
    }
    subscribe(topic, handler, origin) {
        const { handlerId } = this.subscriptions.add(topic, handler);
        const msg = {
            type: "subscribe",
            topic,
            messageOrigin: origin !== null && origin !== void 0 ? origin : this.getUpstreamMessageOrigin(),
            handlerId,
        };
        this.sendOrQueueMessageToSubject(msg);
    }
    unsubscribe(topic, handler, origin) {
        this.subscriptions.delete(topic, handler);
        if (this.subscriptions.isEmpty(topic)) {
            const msg = {
                type: "unsubscribe",
                topic,
                messageOrigin: origin !== null && origin !== void 0 ? origin : this.getUpstreamMessageOrigin(),
            };
            this.sendOrQueueMessageToSubject(msg);
        }
    }
    log(logData) {
        const logMsg = createLogMessage(logData, this.addContextToLogger(), this.getUpstreamMessageOrigin());
        this.sendOrQueueMessageToSubject(logMsg);
    }
    sendLogMessage(message) {
        if (message.type !== "log") {
            this.logger.error("Attempted to send invalid log message", {
                message,
            });
            return;
        }
        message.context = Object.assign(Object.assign({}, message.context), this.addContextToLogger());
        this.sendOrQueueMessageToSubject(message);
    }
    sendMetric({ metricData, time, namespace }) {
        const metricMessage = createMetricMessage({
            metricData,
            time,
            namespace,
        }, this.getUpstreamMessageOrigin());
        this.sendOrQueueMessageToSubject(metricMessage);
    }
    sendMetricMessage(metricMessage) {
        if (metricMessage.type !== "metric") {
            this.logger.error("Attempted to send invalid metric message", {
                metricMessage,
            });
            return;
        }
        this.sendOrQueueMessageToSubject(metricMessage);
    }
    sendOrQueueMessageToSubject(message) {
        if (this.connectionEstablished) {
            this.sendMessageToSubject(message);
        }
        else {
            this.upstreamMessageQueue.push(message);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    consumerMessageHandler(evt) {
        if (!this.isInitialized) {
            this.logger.error("Attempted to process message from subject prior to proxy being initializing. Message not processed", { originalMessageEventData: evt.data });
            return;
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        const { data } = evt;
        if (!("type" in data)) {
            // TODO Clean this up... probably safe to ignore without logging
            this.logger.warn("Unknown inbound message", {
                originalMessageEventData: data,
            });
            return;
        }
        // Naming of type confusing because outbound to worker is inbound to client
        const msg = data;
        this.handleMessageFromSubject(msg);
    }
    handleMessageFromSubject(msg) {
        this.handleDefaultMessageFromSubject(msg);
    }
    handleDefaultMessageFromSubject(msg) {
        switch (msg.type) {
            case "acknowledge":
                this.handleConnectionAcknowledge(msg);
                break;
            case "response":
                this.handleResponse(msg);
                break;
            case "publish":
                this.handlePublish(msg);
                break;
            case "error":
                this.handleError(msg);
                break;
            case "childDownstreamMessage":
                this.channelManager.handleDownstreamMessage(msg);
                break;
            case "childConnectionClose":
                this.channelManager.handleCloseMessage(msg);
                break;
            case "healthCheckResponse":
                this.healthCheck.handleResponse(msg);
                break;
            default:
                this.logger.error("Unknown inbound message", {
                    originalMessageEventData: msg,
                });
                return;
        }
    }
    handleConnectionAcknowledge(msg) {
        this.connectionId = msg.connectionId;
        this.status.update({
            status: "ready",
            connectionId: msg.connectionId,
        });
        this.connectionEstablished = true;
        // Sends any messages in queue
        while (this.upstreamMessageQueue.length) {
            const msg = this.upstreamMessageQueue.shift();
            this.sendMessageToSubject(msg);
        }
        this.healthCheck.start(msg);
    }
    handleResponse(msg) {
        this.requestManager.processResponse(msg);
    }
    handlePublish(msg) {
        const { handlerId, topic } = msg;
        if (handlerId) {
            const handler = this.subscriptions.getById(topic, handlerId);
            if (handler) {
                void this.handleAsyncSubscriptionHandlerInvoke({ handler, handlerId }, msg);
            }
        }
        else {
            this.subscriptions
                .get(topic)
                .map((handlerIdMapping) => void this.handleAsyncSubscriptionHandlerInvoke(handlerIdMapping, msg));
        }
    }
    handleError(msg) {
        if (msg.isFatal) {
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            const { message: reason, type: _ } = msg, details = __rest(msg, ["message", "type"]);
            this.status.update({ status: "error", reason: reason, details });
        }
        this.publishError({
            message: msg.message,
            key: msg.key,
            details: msg.details,
            isFatal: msg.isFatal,
            proxyStatus: msg.status,
        });
    }
    publishError(error) {
        const fullError = Object.assign(Object.assign({}, error), { connectionStatus: this.connectionStatus });
        this.errorService.invoke(fullError);
    }
    handleAsyncSubscriptionHandlerInvoke(_a, _b) {
        return __awaiter(this, arguments, void 0, function* ({ handler, handlerId }, { topic, data }) {
            try {
                yield handler(data);
            }
            catch (error) {
                this.logger.error("An error occurred when handling subscription", {
                    topic,
                    error,
                    handlerId,
                });
            }
        });
    }
    get connectionStatus() {
        return this.status.getStatus();
    }
    onError(handler) {
        this.errorService.onError(handler);
    }
    offError(handler) {
        this.errorService.offError(handler);
    }
    onConnectionStatusChange(handler) {
        this.status.onChange(handler);
    }
    offConnectionStatusChange(handler) {
        this.status.offChange(handler);
    }
    onHealthCheckStatusChanged(handler) {
        this.healthCheck.onStatusChanged(handler);
    }
    offHealthCheckStatusChanged(handler) {
        this.healthCheck.offStatusChanged(handler);
    }
    addChildChannel(params) {
        this.channelManager.addChannel(params);
    }
    updateChildChannelPort(params) {
        this.channelManager.updateChannelPort(params);
    }
    resetConnection(reason) {
        var _a;
        this.connectionEstablished = false;
        this.status.update({
            status: "reset",
            reason,
        });
        const subscriptionHandlerIds = this.subscriptions.getAllSubscriptionHandlerIds();
        this.logger.info("Resetting proxy", {
            reason,
            subscriptionHandlerCount: (_a = subscriptionHandlerIds === null || subscriptionHandlerIds === void 0 ? void 0 : subscriptionHandlerIds.length) !== null && _a !== void 0 ? _a : -1,
        });
        // Restore all subscriptions
        subscriptionHandlerIds === null || subscriptionHandlerIds === void 0 ? void 0 : subscriptionHandlerIds.map(({ topic, handlerId }) => ({
            type: "subscribe",
            topic,
            messageOrigin: this.getUpstreamMessageOrigin(),
            handlerId,
        })).forEach((msg) => this.sendOrQueueMessageToSubject(msg));
        // TODO Notify Connection has repaired
    }
}
//# sourceMappingURL=proxy.js.map