"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChannelManager = void 0;
const logging_1 = require("../logging");
const messaging_1 = require("../messaging");
class ChannelManager {
    constructor(provider, relayChildUpstreamMessage) {
        this.provider = provider;
        this.relayChildUpstreamMessage = relayChildUpstreamMessage;
        this.messagePorts = new Map();
        this.logger = new logging_1.ConnectLogger({
            provider,
            source: "childConnectionManager",
        });
    }
    addChannel(params) {
        const { connectionId } = params;
        if (this.messagePorts.has(connectionId)) {
            this.logger.error("Attempted to add child connection that already exists. No action", {
                connectionId,
            });
            return;
        }
        this.setupPort(params);
        this.logger.debug("Child port added", { connectionId });
    }
    updateChannelPort(params) {
        var _a;
        const { connectionId } = params;
        const originalPort = (_a = this.messagePorts.get(connectionId)) === null || _a === void 0 ? void 0 : _a.port;
        if (!originalPort) {
            this.logger.error("Attempted to update child connection that does not exist No action", {
                connectionId,
            });
            return;
        }
        originalPort.onmessage = null;
        originalPort.close();
        this.setupPort(params);
        this.logger.info("Updated child port", { connectionId });
    }
    setupPort({ connectionId, port, providerId, }) {
        const handler = this.createMessageHandler(connectionId, providerId);
        port.addEventListener("message", handler);
        port.start();
        this.messagePorts.set(connectionId, { port, handler, providerId });
        this.relayChildUpstreamMessage({
            type: "childUpstream",
            connectionId,
            sourceProviderId: providerId,
            parentProviderId: this.provider.id,
            message: {
                type: "childConnectionReady",
            },
        });
    }
    handleDownstreamMessage({ connectionId, message, targetProviderId, }) {
        const messagePortData = this.messagePorts.get(connectionId);
        if (!messagePortData) {
            this.logger.warn("Attempted to route downstream message to child message port that does not exist", { connectionId, message: (0, messaging_1.sanitizeDownstreamMessage)(message) });
            return;
        }
        const { port, providerId } = messagePortData;
        // Older versions of the SDK do not provide a provider id. This
        // check is ignored for versions without a providerId.
        if (providerId && providerId !== targetProviderId) {
            this.logger.error("Downstream target message did not match target provider id. Not sending message.", {
                connectionId,
                targetProviderId,
                actualProviderId: providerId,
                message: (0, messaging_1.sanitizeDownstreamMessage)(message),
            });
            return;
        }
        port.postMessage(message);
    }
    handleCloseMessage({ connectionId }) {
        const messagePortData = this.messagePorts.get(connectionId);
        if (!messagePortData) {
            this.logger.warn("Attempted to close child message port that was not found", { connectionId });
            return;
        }
        const { port, handler } = messagePortData;
        port.removeEventListener("message", handler);
        port.close();
        this.messagePorts.delete(connectionId);
        this.logger.debug("Removed child message channel", { connectionId });
    }
    createMessageHandler(connectionId, providerId) {
        return (message) => this.relayChildUpstreamMessage({
            type: "childUpstream",
            sourceProviderId: providerId,
            parentProviderId: this.provider.id,
            connectionId,
            message: message.data,
        });
    }
}
exports.ChannelManager = ChannelManager;
//# sourceMappingURL=channel-manager.js.map